--     Calendar Buddy - Event sign-ups export for MK discord bot
--     Copyright (C) 2020  Nivix
-- 
--     This program is free software: you can redistribute it and/or modify
--     it under the terms of the GNU General Public License as published by
--     the Free Software Foundation, either version 3 of the License, or
--     (at your option) any later version.
-- 
--     This program is distributed in the hope that it will be useful,
--     but WITHOUT ANY WARRANTY; without even the implied warranty of
--     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--     GNU General Public License for more details.
-- 
--     You should have received a copy of the GNU General Public License
--     along with this program.  If not, see <https://www.gnu.org/licenses/>.

CalendarBuddy = { };

CalendarBuddy.frame = CreateFrame("Frame");
CalendarBuddy.events = { };
CalendarBuddy.options = 
{
    ["sorting"] = 
    { 
        ["default"] = true,
        ["heroic"] = true,
        ["heroic_avg"] = true,
        ["mythic"] = true,
        ["mythic_avg"] = true
    },
    ["timeframe"] =
    { 
        ["today"] = true,
        ["historical"] = true
    },
    ["role"] =
    { 
        ["all"] = true,
        ["hps"] = true,
        ["dps"] = true,
        ["tank"] = true
    },
    ["partition"] = 
    {
        ["latest"] = true,
        ["1"] = true,
        ["2"] = true,
        ["3"] = true,
        ["4"] = true
    }
};

local CalBud = CalendarBuddy;
local events = CalBud.events;
local CalBudFrame = nil;

SLASH_CALBUD1 = "/calbud"

-- Mostly copied from Simulationcraft addon
function CalBud:GetMainFrame(text)
-- Frame code largely adapted from https://www.wowinterface.com/forums/showpost.php?p=323901&postcount=2
    -- Main Frame
    if not CalBudFrame then
        local f = CreateFrame("Frame", "CalBudFrame", UIParent, "DialogBoxFrame");
        f:ClearAllPoints();
        -- load position from local DB
        f:SetPoint("CENTER", nil, "CENTER", 0, 0);
        f:SetSize(750, 400);
        f:SetBackdrop({
            bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",
            edgeFile = "Interface\\PVPFrame\\UI-Character-PVP-Highlight",
            edgeSize = 16,
            insets = { left = 8, right = 8, top = 8, bottom = 8 },
        });
        f:SetMovable(true);
        f:SetClampedToScreen(true);
        f:SetScript("OnMouseDown", function(self, button)
            if button == "LeftButton" then
                self:StartMoving();
            end
        end);
        f:SetScript("OnMouseUp", function(self, button)
            self:StopMovingOrSizing();
        end);

        -- scroll frame
        local sf = CreateFrame("ScrollFrame", "CalBudFrame", f, "UIPanelScrollFrameTemplate");
        sf:SetPoint("LEFT", 16, 0);
        sf:SetPoint("RIGHT", -32, 0);
        sf:SetPoint("TOP", 0, -32);
        sf:SetPoint("BOTTOM", CalBudFrameButton, "TOP", 0, 0);

        -- edit box
        local eb = CreateFrame("EditBox", "CalBudEditBox", CalBudScrollFrame);
        eb:SetSize(sf:GetSize());
        eb:SetMultiLine(true);
        eb:SetAutoFocus(true);
        eb:SetFontObject("ChatFontNormal");
        eb:SetScript("OnEscapePressed", function() f:Hide(); end);
        sf:SetScrollChild(eb);

        -- resizing
        f:SetResizable(true);
        if f.SetMinResize then
            -- older function from shadowlands and before
            -- Can remove when Dragonflight is in full swing
            f:SetMinResize(150, 100)
        else
            -- new func for dragonflight
            f:SetResizeBounds(150, 100, nil, nil)
        end
        local rb = CreateFrame("Button", "CalBudResizeButton", f);
        rb:SetPoint("BOTTOMRIGHT", -6, 7);
        rb:SetSize(16, 16);

        rb:SetNormalTexture("Interface\\ChatFrame\\UI-ChatIM-SizeGrabber-Up");
        rb:SetHighlightTexture("Interface\\ChatFrame\\UI-ChatIM-SizeGrabber-Highlight");
        rb:SetPushedTexture("Interface\\ChatFrame\\UI-ChatIM-SizeGrabber-Down");

        rb:SetScript("OnMouseDown", function(self, button)
            if button == "LeftButton" then
                f:StartSizing("BOTTOMRIGHT");
                self:GetHighlightTexture():Hide(); -- more noticeable
            end
        end)
        rb:SetScript("OnMouseUp", function(self, button)
            f:StopMovingOrSizing();
            self:GetHighlightTexture():Show();
            eb:SetWidth(sf:GetWidth());
        end)
        CalBudFrame = f;
    end
    CalBudEditBox:SetText(text);
    CalBudEditBox:HighlightText();
    return CalBudFrame;
end

SlashCmdList["CALBUD"] = function(msg, editbox)
    local parameter, value = msg:match("^(%S*)%s*(.-)$");
    if msg == "" then
        if CalBud.myRealm == nil then
            CalBud.myRealm = GetNormalizedRealmName();
        end
        local frame = CalBud:GetMainFrame(CalBud:eventString());
        frame:Show();
        return;
    end
    local parameter, value = msg:match("^(%S*)%s*(.-)$");
    if parameter == "reset" then
        CalBud.vars.sorting = "default";
        CalBud.vars.timeframe = "historical";
        CalBud.vars.role = "all";
        CalBud.vars.zone = "recent";
        CalBud.vars.partition = "latest";
        print("CalBud: default values have been reset.");
    elseif CalBud.options[parameter] ~= nil and CalBud.options[parameter][value] ~= nil then
        CalBud.vars[parameter] = value;
        print("CalBud: default " .. parameter .. " is now set to " .. value .. ".");
    else 
        print("Calendar Buddy command list:\n/calbud\n/calbud reset\n/calbud sorting <default, heroic, heroic_avg, mythic, mythic_avg>\n/calbud timeframe <historical, today>\n/calbud role <all, dps, hps, tank>\n/calbud zone <recent, nya, ep, cos, bod, uldir>\n/calbud partition <latest, 1, 2, 3>");
    end
end

function CalBud:event()
    local eventInfo = C_Calendar.GetEventInfo();
    if eventInfo == nil then
        return nil;
    end
    local event = { };
    event.info = eventInfo;
    event.count = C_Calendar.GetNumInvites();
    local inviteList = { };
    for idx = 1, event.count do
        inviteList[idx] = C_Calendar.EventGetInvite(idx);
    end
    event.invites = inviteList;
    return event;
end

function CalBud:infoToString(infoTable)
    local time = infoTable.time;
    local weekDay = CALENDAR_WEEKDAY_NAMES[time.weekday];
    local month = CALENDAR_FULLDATE_MONTH_NAMES[time.month];
    local dateString = string.format("%02d:%02d, %s, %d %s %d", time.hour, time.minute, weekDay, time.monthDay, month, time.year);
    return infoTable.title .. " - " .. dateString;
end

function CalBud:inviteToString(invite)
    local inviteString = "";
    local name = invite.name
    if name:find("-") == nil then
        name = name .. "-" .. CalBud.myRealm;
    end
    return name .. " " .. invite.inviteStatus;
end

function CalBud:invitesToString(invites, numInvites)
    local invitesString = CalBud:inviteToString(invites[1]);
    for idx = 2, numInvites do
        invitesString = invitesString .. ", " ..  CalBud:inviteToString(invites[idx], myRealm);
    end
    return invitesString;
end

function CalBud:argumentsString()
    local vars = CalBud.vars;
   return "sorting=" .. vars.sorting .. " timeframe=" .. vars.timeframe .. " zone=recent partition=" .. vars.partition .. " role=" .. vars.role;
end

function CalBud:eventString()
    local event = CalBud:event();
    if event == nil then
        return "Please open an event first";
    end
    return "!wcl " .. CalBud:infoToString(event.info) .. "\n" ..  CalBud:invitesToString(event.invites, event.count) .. "\n" .. CalBud:argumentsString();
end

function CalBud:init()
    if CalendarBuddySavedVariables == nil then
        CalendarBuddySavedVariables = { };
    end
    local SavedVars = CalendarBuddySavedVariables;
    SavedVars.zone = "recent";
    if SavedVars.sorting == nil then
        SavedVars.sorting = "default";
    end
    if SavedVars.timeframe == nil then
        SavedVars.timeframe = "historical";
    end
    if SavedVars.role == nil then
        SavedVars.role = "all";
    end
    if SavedVars.partition == nil then
        SavedVars.partition = "latest";
    end
    CalBud.vars = SavedVars;
end

function events:ADDON_LOADED(addonName)
    if addonName == "CalendarBuddy" then
        CalBud:init();
    end 
end      

function events:handler(event, ...)
	events[event](self, ...);
end

CalBud.frame:SetScript("OnEvent", events.handler); 
CalBud.frame:RegisterEvent("ADDON_LOADED");
